"""
    Apideck

    The Apideck OpenAPI Spec: SDK Optimized  # noqa: E501

    The version of the OpenAPI document: 10.7.0
    Generated by: https://openapi-generator.tech
"""


import sys
import unittest

import apideck
from apideck.model.links import Links
from apideck.model.meta import Meta
from apideck.model.payment import Payment
globals()['Links'] = Links
globals()['Meta'] = Meta
globals()['Payment'] = Payment
from apideck.model.get_payments_response import GetPaymentsResponse


class TestGetPaymentsResponse(unittest.TestCase):
    """GetPaymentsResponse unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testGetPaymentsResponse(self):
        """Test GetPaymentsResponse"""
        # FIXME: construct object with mandatory attributes with example values
        # model = GetPaymentsResponse()  # noqa: E501
        pass


if __name__ == '__main__':
    unittest.main()
